local addon_name, addon = ...

local L = LibStub("AceLocale-3.0"):GetLocale(addon_name)

addon.frame = _G[addon_name]
if not addon.frame then
	addon.frame = CreateFrame("Frame", addon_name, UIParent, BackdropTemplateMixin and "BackdropTemplate")
end

addon.frame:RegisterEvent("ADDON_LOADED")
addon.frame:SetScript("OnEvent", function(self, event, ...)
	if event == "ADDON_LOADED" or event == "PLAYER_LOGIN" then
		if event == "ADDON_LOADED" then
			local arg1 = ...
			if arg1 and arg1 == addon_name then
				self:UnregisterEvent("ADDON_LOADED")
				self:RegisterEvent("PLAYER_LOGIN")
			end
			return
		end
		if IsLoggedIn() then
			self:UnregisterEvent("PLAYER_LOGIN")
			addon:Initialize()
		end
		return
	end
	if addon[event] then
		addon[event](addon, event, ...)
	end
end)

function addon:RegisterEvent(event)
    self.frame:RegisterEvent(event)
end

local function LocalizeOption(key, option)
    if option.name == "" then
        option.name = L["option_"..key]
    end
    if option.desc == "" then
        option.desc = L["option_"..key.."_desc"]
    end
end
local function ExtractOptionDefaults(db, options, option_defaults)
    for key, option in pairs(options.args) do
        LocalizeOption(key, option)
        if option.default then
            option_defaults[key] = option.default
            if db[key] == nil then db[key] = option.default end
            option.default = nil
        end
        if option.args then
            ExtractOptionDefaults(db, option, option_defaults)
        end
    end
end

function addon:BuildOptions(options)
    self.option_defaults = {}
    ExtractOptionDefaults(self.db, options, self.option_defaults)

    options.handler = self
    options.get = "GetOption"
    options.set = "SetOption"
    LibStub("AceConfig-3.0"):RegisterOptionsTable(addon_name, options)
    LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addon_name, L.addon_name)
end

function addon:GetOption(info)
    local optionName = info[#info]
    if self.db[optionName] ~= nil then
        return self.db[optionName]
    end
    if self.option_defaults[optionName] ~= nil then
        return self.option_defaults[optionName]
    end
    return nil
end
function addon:SetOption(info, value)
    local optionName = info[#info]
    self.db[optionName] = value
    local event = "OPTION_"..optionName
    if self[event] then
        self[event](self)
    end
end
