local addon_name, addon = ...

local anchors = { "TOPLEFT", "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT" }
local strata = { "BACKGROUND", "LOW", "MEDIUM", "HIGH", "DIALOG", "TOOLTIP" }
local styles = { "Tooltip", "DialogBox", "Slick", "None" }

local titleHeight = 4

--------------------------------------------------------------------------------
-- Options

local function RangeOption(order, min, max, default)
	return {
        name = "",
		order = order,
		type = "range",
		min = 0, max = 2000, softMin = min, softMax = max, step = 1,
		default = default
	}
end

local options = {
    type = "group",
    args = {
        style = {
            name = "",
            type = "select",
            order = 1,
            values = styles,
            default = 1
        },
        width = RangeOption(2, 100, 1000, 300),
        toggleTest = {
            name = "",
            type = "execute",
            order = 3,
            func = "ToggleTest"
        },
        anchor = {
            name = "",
            desc = "",
            type = "select",
            order = 4,
            values = anchors,
            default = 4
        },
        anchorX = RangeOption(5, 0, 500, 97),
		anchorY = RangeOption(6, 0, 500, 240),
        frameStrata = {
            name = "",
            type = "select",
            order = 7,
            values = strata,
            default = 5
        },
        backgroundColor = {
            name = "",
            type = "color",
            order = 8,
            hasAlpha = true,
            default = { 0, 0, 0, 0.85 },
            get = function (info) local c = addon:GetOption(info); return c[1], c[2], c[3], c[4] end,
            set = function (info, r, g, b, a) addon:SetOption(info, { r, g, b, a }) end
        },
        -- fontScale = {
        --     name = "",
        --     order = 9,
        --     type = "range",
        --     min = 1, max = 3, softMin = 0.5, softMax = 5,
        --     default = 1
        -- },
        
        notesHeader = {
            name = "",
            type = "header",
            order = 10
        },
        emptyLines = {
            name = "",
            desc = "",
            type = "toggle",
            order = 11,
            default = true
        },
        showInterrupts = {
            name = "",
            type = "toggle",
            order = 12,
            default = true
        },
        showPurges = {
            name = "",
            type = "toggle",
            order = 13,
            default = true
        }
    }
}

function addon:OPTION_anchor() self:PositionFrame() end
function addon:OPTION_anchorX() self:PositionFrame() end
function addon:OPTION_anchorY() self:PositionFrame() end
function addon:OPTION_style() self:SetFrameStyle() end
function addon:OPTION_width()
    self.frame:SetWidth(self.db.width)
    if self.frame:IsShown() then
        self:FitToContents()
    end
end
function addon:OPTION_frameStrata()
    self.frame:SetFrameStrata(strata[self.db.frameStrata])
end
function addon:OPTION_backgroundColor()
    local style = styles[self.db.style]
	if style == "Tooltip" or style == "Slick" then
        self:SetBackdropColor()
	end
end

function addon:OPTION_emptyLines() self:ShowNoteAgain() end
function addon:OPTION_showInterrupts() self:ShowNoteAgain() end
function addon:OPTION_showPurges() self:ShowNoteAgain() end

--------------------------------------------------------------------------------
-- Frame

function addon:Initialize()
    if not EncounterNotes_DB then EncounterNotes_DB = {} end
    self.db = EncounterNotes_DB
    self:BuildOptions(options)

    self:InitializeData()

    self:BuildFrame()
    self:PositionFrame()
    self:SetFrameStyle()

    self:RegisterEvent("ENCOUNTER_START")
	self:RegisterEvent("ENCOUNTER_END")
end

function addon:BuildFrame()
    local frame = self.frame
    frame:SetFrameStrata(strata[self.db.frameStrata])
    frame:SetWidth(self.db.width)
    frame:SetHeight(100)

    local titleText = frame:CreateFontString(nil, "BACKGROUND")
    titleText:SetFontObject("GameFontNormalLarge")
    titleText:SetJustifyV("TOP")
    titleText:SetJustifyH("LEFT")
    titleText:SetText("Title")
    self.titleText = titleText

    local notesText = frame:CreateFontString(nil, "BACKGROUND")
    notesText:SetFontObject("GameFontNormal")
    notesText:SetJustifyV("TOP")
    notesText:SetJustifyH("LEFT")
    notesText:SetTextColor(1, 1, 1)
    notesText:SetText("Notes")
    self.notesText = notesText

    frame:Hide()
end

function addon:PositionFrame()
    local frame = self.frame
    frame:ClearAllPoints()
    local anchor = anchors[self.db.anchor]
    local x = self.db.anchorX
    if anchor:find("RIGHT", 1, true) then
        x = -x
    end
    local y = self.db.anchorY
    if anchor:find("TOP", 1, true) then
        y = -y
    end
    frame:SetPoint(anchor, x, y)
end

function addon:LayoutFrame()
    local inset = self.inset

    local titleText = self.titleText
    titleText:ClearAllPoints()
    titleText:SetPoint("TOPLEFT", inset, -inset)
    titleText:SetPoint("RIGHT", -inset, 0)
    
    local notesText = self.notesText
    notesText:ClearAllPoints()
    notesText:SetPoint("TOPLEFT", inset, -inset - titleHeight)
    notesText:SetPoint("RIGHT", -inset, 0)

    if self.frame:IsShown() then
        self:FitToContents()
    end
end

function addon:SetBackdropColor()
    local color = self.db.backgroundColor
    self.frame:SetBackdropColor(color[1], color[2], color[3], color[4])
end
function addon:SetFrameStyle()
    local style = styles[self.db.style]
	if style == "Tooltip" then
        self.inset = 10
        self.frame:SetBackdrop(BACKDROP_TOOLTIP_16_16_5555)
        self:SetBackdropColor()
    elseif style == "DialogBox" then
        self.inset = 16
		self.frame:SetBackdrop(BACKDROP_DIALOG_32_32)
        self.frame:SetBackdropColor(1, 1, 1)
    elseif style == "Slick" then
        self.inset = 6
        self.frame:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16 })
        self:SetBackdropColor()
	else
        self.inset = 0
		self.frame:SetBackdrop(nil)
	end

    self:LayoutFrame()
end

function addon:FitToContents()
    self.frame:SetHeight(titleHeight + self.notesText:GetHeight() + (self.inset * 2))
end

--------------------------------------------------------------------------------
-- Notes

local function GetPlayerTags()
    local tags = {}
    if addon.db.showInterrupts then
        tags.int = true
    end
    if addon.db.showPurges then
        tags.purge = true
    end
    local role = UnitGroupRolesAssigned("player")
    if role == "TANK" then
        tags.tank = true
    elseif role == "DAMAGER" then
        tags.dps = true
    elseif role == "HEALER" then
        tags.heal = true
    end
    return tags
end

function addon:ShowNote(note)
    self.lastShownNote = note
    self.titleText:SetText(note.name)
    local tags = GetPlayerTags()
    local noteItems = { "" }
    for _, item in ipairs(note) do
        if not item.t or (not item.n and tags[item.t]) or (item.n and not tags[item.t]) then
            table.insert(noteItems, item[1])
        end
    end
    self.notesText:SetText(table.concat(noteItems, (self.db.emptyLines and "\n\n" or "\n")))
    self.frame:Show()
    self:FitToContents()
end

function addon:ShowNoteAgain()
    if self.frame:IsShown() and self.lastShownNote then
        self:ShowNote(self.lastShownNote)
    end
end

function addon:ToggleTest()
    local frame = self.frame
    if frame:IsShown() then
        frame:Hide()
    else
        -- Choose random note
        local notes = {}
        for _, n in pairs(self.data) do
            table.insert(notes, n)
        end
        local note = notes[math.random(#notes)]
        self:ShowNote(note)
    end
end

function addon:ENCOUNTER_START(event, ...)
    local encounterID = ...
    local note = self.data[encounterID]
    if note then
        self:ShowNote(note)
    end
end

function addon:ENCOUNTER_END()
    self.frame:Hide()
end